/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.Collection;
import java.util.EventObject;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.conqat.lib.commons.collections.UnmodifiableMap;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.ITutorialProvider;
import org.fortiss.tooling.kernel.extension.ITutorialStepCompletionChecker;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.TutorialDefinition;
import org.fortiss.tooling.kernel.extension.data.TutorialStep;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.ITutorialService;
import org.fortiss.tooling.kernel.service.listener.ITutorialServiceListener;

/**
 * This class implements the {@link ITutorialService} interface.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public final class TutorialService implements ITutorialService, CommandStackListener {
	/** The collection of all defined tutorials. */
	private Map<String, Class<? extends ITutorialProvider>> allTutorials = new HashMap<>();
	/** The collection of listeners. */
	private Collection<ITutorialServiceListener> listeners =
			new LinkedList<ITutorialServiceListener>();

	/** The active tutorial. */
	private ITutorialProvider activeTutorial = null;
	/** The active tutorial step. */
	private TutorialStep activeStep = null;
	/** The active top-level element. */
	private ITopLevelElement activeTopElement = null;

	/** {@inheritDoc} */
	@Override
	public boolean isTutorialActive() {
		return activeTutorial != null;
	}

	/** {@inheritDoc} */
	@Override
	public ITutorialProvider getActiveTutorial() {
		return activeTutorial;
	}

	/** {@inheritDoc} */
	@Override
	public TutorialStep getActiveTutorialStep() {
		return activeStep;
	}

	/** {@inheritDoc} */
	@Override
	public Map<String, Class<? extends ITutorialProvider>> getAllTutorialProviders() {
		return new UnmodifiableMap<String, Class<? extends ITutorialProvider>>(allTutorials);
	}

	/** {@inheritDoc} */
	@Override
	public void registerTutorialProvider(Class<? extends ITutorialProvider> provider,
			String displayName) {
		allTutorials.put(displayName, provider);
	}

	/** {@inheritDoc} */
	@Override
	public void startTutorial(Class<? extends ITutorialProvider> provider) {
		if(!allTutorials.containsValue(provider)) {
			return;
		}
		try {
			activeTutorial = provider.newInstance();
			activeStep = activeTutorial.getDefinition().getSteps().get(0);
			EObject root = activeTutorial.getRootElement();
			activeTopElement = IPersistencyService.INSTANCE.addDummyEObjectAsTopLevelElement(root);
			ICommandStackService.INSTANCE.addCommandStackListener(activeTopElement, this);
		} catch(InstantiationException e) {
			e.printStackTrace();
		} catch(IllegalAccessException e) {
			e.printStackTrace();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void stopTutorial() {
		ICommandStackService.INSTANCE.removeCommandStackListener(activeTopElement, this);
		IPersistencyService.INSTANCE.removeDummyTopLevelElement(activeTutorial.getRootElement());
		activeTopElement = null;
		activeStep = null;
		activeTutorial = null;
	}

	/** {@inheritDoc} */
	@Override
	public void addTutorialServiceListener(ITutorialServiceListener l) {
		if(!listeners.contains(l)) {
			listeners.add(l);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void removeTutorialServiceListener(ITutorialServiceListener l) {
		listeners.remove(l);
	}

	/** {@inheritDoc} */
	@Override
	public boolean prototypeActive(Prototype prototype) {
		return activeTutorial.getGlobalWhitelistProvider().prototypeActive(prototype) ||
				activeStep.getLocalWhitelistProvider().prototypeActive(prototype);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isEarlierStep(TutorialStep step) {
		if(activeStep == TUTORIAL_COMPLETED_STEP) {
			return true;
		}
		TutorialDefinition def = activeTutorial.getDefinition();
		int current = def.getSteps().indexOf(activeStep);
		int test = def.getSteps().indexOf(step);
		return test < current;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isLaterStep(TutorialStep step) {
		if(activeStep == TUTORIAL_COMPLETED_STEP) {
			return false;
		}
		TutorialDefinition def = activeTutorial.getDefinition();
		int current = def.getSteps().indexOf(activeStep);
		int test = def.getSteps().indexOf(step);
		return test > current;
	}

	/** {@inheritDoc} */
	@Override
	public void commandStackChanged(EventObject event) {
		List<TutorialStep> steps = activeTutorial.getDefinition().getSteps();
		activeStep = TUTORIAL_COMPLETED_STEP;
		for(TutorialStep step : steps) {
			if(!step.getCompletionChecker().isTutorialStepComplete(activeTutorial.getRootElement())) {
				activeStep = step;
				break;
			}
		}
	}

	/** The shared hidden final state of all tutorials. */
	private static TutorialStep TUTORIAL_COMPLETED_STEP = new TutorialStep("Tutorial complete.") {
		@Override
		public ITutorialStepCompletionChecker getCompletionChecker() {
			return new ITutorialStepCompletionChecker() {
				@Override
				public boolean isTutorialStepComplete(EObject model) {
					// This step can never be completed.
					return false;
				}
			};
		}
	};
}
